{Demo Application for the Clip Controls (ribbons in the
 caption bar):

 Uses the descendants of TMDIWindow, TWindow, TDialog and
 tDialogWindow (from JanusWindow by Peter Sawatzki,
 CIS ID 100031,3002) for

 - the MDI-Window that has all kinds of possible clips
 - a normal MDI child that displays the clips in the
   menu if it is maximized
 - a dialog as a MDI child using the tDialogWindow
   descendant tCCMDICDialogWindow
 - a normal dialog (tDialog descendant tCCDialog)
 - a 3D-dialog (tDialogWindow descendant tCCDialogWindow)

 Developer: Sebastian Modersohn
            Softwaredevelopment
            Heinestrasse 20, D-15370 Petershagen, Germany
            CIS ID 100340,1474

 Copyright (c) 1994 Sebastian Modersohn. All Rights Reserved.
}
program CCMDI;

{$R CCMDI}

uses WinTypes, WinProcs, OWindows,
     {the unit of tDialogWindow made by P. Sawatzki}
     DialogWn,
     {the units of the 5 descendants}
     MDIClip, MDICClip, DlgClip, JanClip, JanCClip;

const cm_NewWindow       =24331;
      cm_NewDlgChild     =1011;
      cm_CloseWindow     =1006;
      cm_Dialog          =10001;
      cm_Dialog3D        =10002;

      cm_AddClip         =1002;
      cm_MainHelp        =1200;
      cm_Additional      =1010;

      cm_ChildAddClip    =1004;
      cm_ChildAdditional =1110;
      cm_ChildHelp       =1210;
      cm_DlgClip         =3010;

type
  {the MDI-Window object descended from tCCMDIWindow}
  pMainWindow = ^tMainWindow;
  tMainWindow = object (tCCMDIWindow)
    constructor Init(ATitle: PChar; AMenu: HMenu);
    destructor Done; virtual;

    procedure SetupWindow; virtual;

    procedure NewWindow(var Msg: TMessage);
      virtual cm_first + cm_NewWindow;
    procedure NewDlgChild(var Msg: TMessage);
      virtual cm_first + cm_NewDlgChild;
    procedure CloseWindow(var Msg: TMessage);
      virtual cm_first + cm_CloseWindow;
    procedure CMAddClip(var Msg: TMessage);
      virtual cm_first + cm_AddClip;
    procedure CMMainHelp(var Msg: TMessage);
      virtual cm_first + cm_MainHelp;

    procedure Dialog(var Msg: TMessage);
      virtual cm_first + cm_Dialog;
    procedure Dialog3D(var Msg: TMessage);
      virtual cm_first + cm_Dialog3D;
  end;

  {the normal MDI-child descended from tCCMDIChildWindow}
  pChildWindow = ^tChildWindow;
  tChildWindow = object (tCCMDIChildWindow)
    procedure SetupWindow; virtual;

    procedure CMAddClip(var Msg: TMessage);
      virtual cm_first + cm_ChildAddClip;
    procedure CMChildHelp(var Msg: TMessage);
      virtual cm_first + cm_ChildHelp;
  end;

  {the dialog-MDI-child descended from tCCMDICDialogWindow}
  pDlgChildWindow = ^tDlgChildWindow;
  tDlgChildWindow = object (tCCMDICDialogWindow)
    procedure SetupWindow; virtual;

    procedure CMAddClip(var Msg: TMessage);
      virtual cm_first + cm_ChildAddClip;
    procedure CMChildHelp(var Msg: TMessage);
      virtual cm_first + cm_ChildHelp;
  end;

  {the normal dialog descended from tCCDialog}
  pTestDialog = ^tTestDialog;
  tTestDialog = object (tCCDialog)
    procedure SetupWindow; virtual;

    procedure CMChildHelp(var Msg: TMessage);
      virtual cm_first + cm_ChildHelp;
  end;

  {the 3D-dialog descended from tCCDialogWindow}
  pTest2Dialog = ^tTest2Dialog;
  tTest2Dialog = object (tCCDialogWindow)
    procedure SetupWindow; virtual;

    procedure CMChildHelp(var Msg: TMessage);
      virtual cm_first + cm_ChildHelp;
  end;

  {the app-object}
  tApp = object (tApplication)
    procedure InitMainWindow; virtual;
  end;

{Inits the main window and sets the menupos for listing
 the child windows to 2}
constructor tMainWindow.Init(ATitle: PChar; AMenu: HMenu);
begin
  inherited Init(ATitle, AMenu);
{If you want to see that The Clip Controls support all kinds
 of frames and the standard clips, use this line to change
 the main window's style}

{  Attr.Style:=Attr.Style and (not ws_ThickFrame)
               and (not ws_MaximizeBox)
               and (not ws_MinimizeBox) or ws_Border ;}

  ChildMenuPos:=2;
end;

{closes the help and calls the inherited destructor}
destructor tMainWindow.Done;
begin
  WinHelp(HWindow, 'clipcon.hlp', Help_Quit, 0);
  inherited Done;
end;

{the setupwindow method used to init all clips}
procedure tMainWindow.SetupWindow;
begin
  inherited SetupWindow;
  {adds a "sticky" clip which can *not* be disabled}
  AddStickyClip('AddClip', cm_AddClip, False);

  {adds a "sticky" clip with 3 states which can *not* be disabled}
  AddStickyStateClip('3StateClip', 1003, 3, False);

  {adds a normal clip which can *not* be disabled}
  AddClip('CustomText', 1005, False);

  {if you do not want to use the string resources for
  the "tips", this is possible too:}
  SetClipText(3, '"Custom" Text for Clip5');

  {adds a normal clip which can *not* be disabled}
  AddClip('NewWindow', cm_NewWindow, False);

  {adds a normal clip which can be disabled}
  AddClip('CloseWindow', cm_CloseWindow, True);

  {disables the clip with the ID cm_CloseWindow }
  EnableClip(GetClipById(cm_CloseWindow), False);

  {adds a normal clip which can *not* be disabled}
  AddClip('Help', cm_MainHelp, False);

  {adds a normal clip with 2 states which can *not* be disabled}
  AddStateClip('Additional', cm_Additional, 2, False);
end;

{makes a new normal MDI-child and enables the clip and the
 menu item for closing the active MDI-child}
procedure tMainWindow.NewWindow(var Msg: TMessage);
begin
  Application^.MakeWindow(New(pChildWindow, Init(@Self, 'Clip MDI Child')));
  EnableClip(GetClipById(cm_CloseWindow), True);
  EnableMenuItem(GetMenu(HWindow), cm_CloseWindow, mf_ByCommand or mf_Enabled);
end;

{makes a new dialog as a MDI-child}
procedure tMainWindow.NewDlgChild(var Msg: TMessage);
begin
  Application^.MakeWindow(New(pDlgChildWindow, Init(@Self, 'dlg_MDIChild')));
  EnableClip(GetClipById(cm_CloseWindow), True);
  EnableMenuItem(GetMenu(HWindow), cm_CloseWindow, mf_ByCommand or mf_Enabled);
end;

{closes the active MDI-child and, if it was the last child,
 disables the clip and the menu item for closing the active child}
procedure tMainWindow.CloseWindow(var Msg: TMessage);
begin
  SendMessage(LoWord(SendMessage(ClientWnd^.HWindow, wm_MDIGetActive, 0, 0)), wm_Close, 0, 0);
  if not IsWindow(LoWord(SendMessage(ClientWnd^.HWindow, wm_MDIGetActive, 0, 0))) then
  begin
    EnableClip(GetClipById(cm_CloseWindow), False);
    EnableMenuItem(GetMenu(HWindow), cm_CloseWindow, mf_ByCommand or mf_Grayed);
  end;
end;

{"AddClip"-clip was pressed, so Adds or Deletes the
 "Additional"-clip; updates the MDI-menu too}
procedure tMainWindow.CMAddClip(var Msg: TMessage);
begin
  if IsDown(GetClipById(cm_AddClip)) then
  begin
    DelClip(GetClipById(cm_Additional));
    DeleteMenu(GetMenu(HWindow), cm_Additional, mf_ByCommand);
  end else
  begin
    AddStateClip('Additional', cm_Additional, 2, False);
    InsertMenu(GetMenu(HWindow), cm_CloseWindow, mf_ByCommand or mf_String or mf_Grayed, cm_Additional, 'Additional');
  end;
end;

{calls the help for that window}
procedure tMainWindow.CMMainHelp(var Msg: TMessage);
begin
  WinHelp(HWindow, 'clipcon.hlp', Help_Context, 400);
end;

{displays a normal dialog}
procedure tMainWindow.Dialog(var Msg: TMessage);
begin
  Application^.ExecDialog(New(pTestDialog, Init(@Self, 'dlg_clip')));
end;

{displays a 3D-dialog}
procedure tMainWindow.Dialog3D(var Msg: TMessage);
begin
  ExecDialogWindow(New(pTest2Dialog, Init(@Self, 'dlg_3Dclip')));
end;

{add the clips for the normal child window}
procedure tChildWindow.SetupWindow;
begin
  inherited SetupWindow;
  {adds a "sticky" clip which can *not* be disabled}
  AddStickyClip('AddClip', cm_ChildAddClip, False);

  {adds a normal clip which can *not* be disabled}
  AddClip('Help', cm_ChildHelp, False);

  {adds a "sticky" clip with 2 states which can *not* be disabled}
  AddStickyStateClip('Additional', cm_ChildAdditional, 2, False);
end;

{"AddClip"-clip was pressed, so Adds or Deletes the
 "Additional"-clip}
procedure tChildWindow.CMAddClip(var Msg: TMessage);
begin
  if IsDown(GetClipById(cm_ChildAddClip)) then
  begin
    DelClip(GetClipById(cm_ChildAdditional));
  end else
  begin
    AddStickyStateClip('Additional', cm_ChildAdditional, 2, False);
  end;
end;

{calls the help for that window}
procedure tChildWindow.CMChildHelp(var Msg: TMessage);
begin
  WinHelp(Application^.MainWindow^.HWindow, 'clipcon.hlp', Help_Context, 300);
end;

{add the clips for the dlg-child-window}
procedure tDlgChildWindow.SetupWindow;
begin
  inherited SetupWindow;
  {adds a "sticky" clip which can *not* be disabled}
  AddStickyClip('AddClip', cm_ChildAddClip, False);

  {adds a normal clip which can *not* be disabled}
  AddClip('Help', cm_ChildHelp, False);

  {adds a "sticky" clip with 2 states which can *not* be disabled}
  AddStickyStateClip('Additional', cm_ChildAdditional, 2, False);
end;

{"AddClip"-clip was pressed, so Adds or Deletes the
 "Additional"-clip}
procedure tDlgChildWindow.CMAddClip(var Msg: TMessage);
begin
  if IsDown(GetClipById(cm_ChildAddClip)) then
  begin
    DelClip(GetClipById(cm_ChildAdditional));
  end else
  begin
    AddStickyStateClip('Additional', cm_ChildAdditional, 2, False);
  end;
end;

{calls the help for that window}
procedure tDlgChildWindow.CMChildHelp(var Msg: TMessage);
begin
  WinHelp(Application^.MainWindow^.HWindow, 'clipcon.hlp', Help_Context, 200);
end;

{add the clips for the normal dialog}
procedure tTestDialog.SetupWindow;
begin
  inherited SetupWindow;
  AddClip('Help', cm_ChildHelp, False);
  AddStateClip('Additional', cm_Additional, 2, False);
end;

{calls the help for that window}
procedure tTestDialog.CMChildHelp(var Msg: TMessage);
begin
  WinHelp(Application^.MainWindow^.HWindow, 'clipcon.hlp', Help_Context, 500);
end;

{add the clips for the 3D-dialog}
procedure tTest2Dialog.SetupWindow;
begin
  inherited SetupWindow;
  AddClip('Help', cm_ChildHelp, False);
  AddStateClip('Additional', cm_Additional, 2, False);
end;

{calls the help for that window}
procedure tTest2Dialog.CMChildHelp(var Msg: TMessage);
begin
  WinHelp(Application^.MainWindow^.HWindow, 'clipcon.hlp', Help_Context, 100);
end;

procedure tApp.InitMainWindow;
begin
  MainWindow:=New(pMainWindow, Init('Clip Control MDI-Demo', LoadMenu(hInstance, 'MNU_MAIN')));
end;

var App: tApp;

begin
  App.Init('CCDemo');
  App.Run;
  App.Done;
end.