Option Explicit
'
' User Defined Types
'
  '
  ' Used for GetCursor - gets mouse location in screen coordinates.
  '
    Type POINTAPI
      X As Integer
      Y As Integer
    End Type
  '
  ' Used by WM_SYSCOMMAND - converts mouse location.
  '
    Type ConvertPOINTAPI
      xy As Long
    End Type
  '
  ' .INI File Type
  '
    Type INI_FILE_TYPE
      fTop As Single
      fLeft As Single
      nStyle As Integer
      lColor As Long
    End Type
'
' API Calls
'
  '
  ' Send Windows Message
  '
    Declare Function SendmessageByNum Lib "User" Alias "SendMessage" (ByVal hWnd As Integer, ByVal wMsg As Integer, ByVal wParam As Integer, ByVal lParam As Long) As Integer
  '
  ' Get Cursor Position
  '
    Declare Sub GetCursorPos Lib "User" (lpPoint As POINTAPI)
  '
  ' Set Window Position
  '
    Declare Function SetWindowPos Lib "User" (ByVal hWnd As Integer, ByVal hWndInsertAfter As Integer, ByVal X As Integer, ByVal Y As Integer, ByVal cx As Integer, ByVal cy As Integer, ByVal wFlags As Integer) As Integer
  '
  ' .INI File Functions
  '
    Declare Function GetPrivateProfileString Lib "Kernel" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Integer, ByVal lpFileName As String) As Integer
    Declare Function WritePrivateProfileString Lib "Kernel" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpString As String, ByVal lplFileName As String) As Integer
'
' Module Constants
'
  '
  ' For SetWindowPos API Call
  '
    Const SWP_NOMOVE = 2
    Const SWP_NOSIZE = 1
    Const FLAGS = SWP_NOMOVE Or SWP_NOSIZE
    Const HWND_TOPMOST = -1
    Const HWND_NOTOPMOST = -2
    Const HWND_BOTTOM = 1
    Const HWND_TOP = 0
  '
  ' .INI file constants
  '
    Const INI_FILENAME = "SCRNTEST.INI"
    Const MAX_INI_STRING = 255
'
' Global Constants
'
  '
  ' For Window Movement API calls
  '
    Global Const WM_LBUTTONUP = &H202
    Global Const WM_SYSCOMMAND = &H112
    Global Const MOUSE_MOVE = &HF012
  '
  ' Standard VB Keyboard Constants
  '
    Global Const ALT_MASK = 4
    Global Const KEY_F4 = &H73
    Global Const KEY_LBUTTON = &H1
    Global Const KEY_RBUTTON = &H2
  '
  ' Standard VB WindowState Constant
  '
    Global Const NORMAL = 0
    Global Const MINIMIZED = 1
  '
  ' Screen Size Constants
  '
    Global Const SCR_VGA_HEIGHT = 7605
    Global Const SCR_VGA_WIDTH = 9720
    Global Const SCR_SVGA_HEIGHT = 9405
    Global Const SCR_SVGA_WIDTH = 12120
    Global Const SCR_1024_HEIGHT = 11580
    Global Const SCR_1024_WIDTH = 15480
  '
  ' Style Constants
  '
    Global Const STYLE_VGA = 1
    Global Const STYLE_SVGA = 2
    Global Const STYLE_1024 = 3
'
' Module Variables
'
  '
  ' Throwaway Return variable
  '
    Dim r As Variant
  '
  ' INI variable
  '
    Dim muINIVals As INI_FILE_TYPE

Sub ExitProgram ()
  '
  ' Centralized Exit from program
  '
  If frmMain.WindowState = MINIMIZED Then
    frmMain.WindowState = NORMAL
  End If
  SaveINIValues
  End
End Sub

Sub GetINIValues ()
  '
  ' Gets INI values from File
  '
  Dim nSize As Integer
  Dim sReturnString As String
  '
  ' Get Form Top Value
  '
  sReturnString = String$(MAX_INI_STRING, 32)
  nSize = GetPrivateProfileString("Screen Tester", "Top", "300", sReturnString, MAX_INI_STRING, INI_FILENAME)
  muINIVals.fTop = Val(Mid$(sReturnString, 1, nSize))
  '
  ' Get Form Left Value
  '
  sReturnString = String$(MAX_INI_STRING, 32)
  nSize = GetPrivateProfileString("Screen Tester", "Left", "300", sReturnString, MAX_INI_STRING, INI_FILENAME)
  muINIVals.fLeft = Val(Mid$(sReturnString, 1, nSize))
  '
  ' Get Form Style Value
  '
  sReturnString = String$(MAX_INI_STRING, 32)
  nSize = GetPrivateProfileString("Screen Tester", "Style", "1", sReturnString, MAX_INI_STRING, INI_FILENAME)
  muINIVals.nStyle = Val(Mid$(sReturnString, 1, nSize))

  '
  ' Get Form Color Value
  '
  sReturnString = String$(MAX_INI_STRING, 32)
  nSize = GetPrivateProfileString("Screen Tester", "Color", "0", sReturnString, MAX_INI_STRING, INI_FILENAME)
  muINIVals.lColor = Val(Mid$(sReturnString, 1, nSize))
End Sub

Sub Main ()
  '
  ' Start of program
  '
  GetINIValues
  Load frmUtility
  frmMain.Show
End Sub

Sub SaveINIValues ()
  '
  ' Write .INI Values
  '
  r = WritePrivateProfileString("Screen Tester", "Top", frmMain.Top, INI_FILENAME)
  r = WritePrivateProfileString("Screen Tester", "Left", frmMain.Left, INI_FILENAME)
  r = WritePrivateProfileString("Screen Tester", "Style", CStr(muINIVals.nStyle), INI_FILENAME)
  r = WritePrivateProfileString("Screen Tester", "Color", frmMain.BackColor, INI_FILENAME)
End Sub

Sub SendFormtoBack (frm As Form)
  '
  ' Places form at the lowest in the Zorder
  '
  r = SetWindowPos(frm.hWnd, HWND_BOTTOM, 0, 0, 0, 0, FLAGS)
End Sub

Sub SetStyle (frm As Form, nStyle As Integer)
  '
  ' Sets the style of the form (VGA, SVGA, 1024) and saves it
  '
  Select Case nStyle
    Case STYLE_VGA
      frm.Height = SCR_VGA_HEIGHT
      frm.Width = SCR_VGA_WIDTH
    Case STYLE_SVGA
      frm.Height = SCR_SVGA_HEIGHT
      frm.Width = SCR_SVGA_WIDTH
    Case STYLE_1024
      frm.Height = SCR_1024_HEIGHT
      frm.Width = SCR_1024_WIDTH
  End Select
  muINIVals.nStyle = nStyle
End Sub

Sub SetUpForm (frm As Form)
  '
  ' Sets form up based on .INI file info
  '
  frm.Move muINIVals.fLeft, muINIVals.fTop
  frm.BackColor = muINIVals.lColor
  SetStyle frm, muINIVals.nStyle
End Sub

