{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

{  File name     :  StrObj.Pas
   Object        :  TStringObject
   Author        :  Ian Hayes
   Copyright     :  ICFM Software, London, UK
                    CIS [100010,1415]
   Version       :  1.0
   Release Date  :  5th January,1995
   Documentation :  see STROBJ.WRI
}

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

UNIT StrObj;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

{***$DEFINE UseStreams}
{$UNDEF UseStreams}

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

INTERFACE

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

USES

   WinTypes,
   WinProcs,
   Objects;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

CONST

   {$IFDEF UseStreams}
   IStringObject = ????;   { <== SET THIS stream id CONST if using STREAMS }
   {$ENDIF}

   { case usage constants }

   Ignore_Case = TRUE;
   Use_Case    = FALSE;

   { string comparison function integer results }

   Compare_LT = -1;
   Compare_EQ = 0;
   Compare_GT = 1;

   Ignore_Len = 0;

   { boolean to string conversion types }

   bt_YesNo     = 1;
   bt_10        = 2;
   bt_TrueFalse = 3;
   bt_TF        = 4;
   bt_YN        = 5;
   bt_OnOff     = 6;
   bt_Last      = 6;

   { special processing flags for delimiter parsing actions }

   delim_None   = 0;
   delim_IncDIC = 1;
   delim_IncSIC = 2;
   delim_ExcDIC = 4;
   delim_ExcSIC = 8;

   WordResError : WORD = $FFFF;

TYPE

   PStringObject = ^TStringObject;

   TStringObject = OBJECT(TObject)
      PRIVATE
         TheStr     : PChar;
         TheMaxSize : WORD;
         TheLength  : WORD;
         TheSizeInc : WORD;
         FUNCTION AssignString(S : STRING) : PChar;
         FUNCTION CanCat(NewStr : PChar; VAR Extra : WORD) : BOOLEAN;
         FUNCTION CanCopy(NewStr : PChar; VAR Extra : WORD) : BOOLEAN;
         FUNCTION ChkSizeInc(E : WORD) : WORD;
         FUNCTION CompConv(I : INTEGER) : INTEGER;
         PROCEDURE DisposeStr;
         FUNCTION ExpandBy(ExtraLen : WORD) : BOOLEAN;
         PROCEDURE InitDataMembers;
         PROCEDURE SetBufferLen(NewLen : WORD);
      PUBLIC
         { *** CONSTRUCTORS *** }
         CONSTRUCTOR InitDefault;
         CONSTRUCTOR InitSize(AMaxSize: WORD; AStr: PChar);
         CONSTRUCTOR Init(AStr: PChar);
         CONSTRUCTOR InitNL(AStr: PChar);
         CONSTRUCTOR InitString(S : STRING);
         CONSTRUCTOR InitBoolean(B : BOOLEAN; StrType : INTEGER);
         CONSTRUCTOR InitExt(E : EXTENDED; Width,Places : BYTE);
         CONSTRUCTOR InitLong(L : LONGINT);
         CONSTRUCTOR InitPtr(P : POINTER);
         CONSTRUCTOR InitReal(R : REAL; Width,Places : BYTE);
         { *** DESTRUCTOR *** }
         DESTRUCTOR Done; VIRTUAL;
         { *** STREAM RELATED *** }
         {$IFDEF UseStreams}
         CONSTRUCTOR Load(VAR S : TStream);
         PROCEDURE Store(VAR S : TStream); VIRTUAL;
         {$ENDIF}
         { *** DATA MEMBER RELATED *** }
         FUNCTION GetLength : WORD;
         FUNCTION GetPChar : PChar;
         FUNCTION GetString : STRING;
         PROCEDURE SetSizeInc(ASize : WORD);
         { *** OPERATIONS WITH OTHER TStringObjects *** }
         FUNCTION AppendO(Other : PStringObject) : PChar;
         FUNCTION AssignO(Other : PStringObject) : PChar;
         FUNCTION CompareO(Other : PStringObject; IgnoreCase : BOOLEAN; Len : WORD) : INTEGER;
         FUNCTION IsSameO(Other : PStringObject) : BOOLEAN;
         FUNCTION IncludesO(Other : PStringObject) : BOOLEAN;
         FUNCTION PrependO(Other : PStringObject) : PChar;
         FUNCTION WithinO(Other : PStringObject) : BOOLEAN;
         { *** PChar related METHODS *** }
         FUNCTION Append(NewStr : PChar) : PChar;
         FUNCTION Append2(Part1,Part2 : PChar) : PChar;
         FUNCTION AppendLen(Source : PChar; Len : WORD) : PChar;
         FUNCTION AppendMid(Source : PChar; Start,Count : WORD) : PChar;
         FUNCTION AppendPad(Source : PChar; Len : WORD; Ch : CHAR) : PChar;
         FUNCTION AppendNL(NewStr : PChar) : PChar;
         FUNCTION AppendSIC(Source : PChar) : PChar;
         FUNCTION AppendDIC(Source : PChar) : PChar;
         FUNCTION AppendTrim(Source : PChar) : PChar;
         FUNCTION Assign(NewStr : PChar) : PChar;
         FUNCTION AssignAppend(Source,Cat : PChar) : PChar;
         FUNCTION AssignAppend2(Source,Cat1,Cat2 : PChar) : PChar;
         FUNCTION AssignLen(NewStr : PChar; Len : WORD) : PChar;
         FUNCTION AssignMid(Source : PChar; Start,Count : WORD) : PChar;
         FUNCTION AssignPad(Source : PChar; Len : WORD; Ch : CHAR) : PChar;
         FUNCTION AssignNL(NewStr : PChar) : PChar;
         FUNCTION AssignTrim(Source : PChar) : PChar;
         FUNCTION Prepend(NewStr : PChar) : PChar;
         { *** Character/array element related *** }
         FUNCTION FirstCharIs(Ch : CHAR) : BOOLEAN;
         FUNCTION FirstNonSpaceChar(VAR Ch : CHAR) : WORD;
         FUNCTION GetChar(w : WORD) : CHAR;
         FUNCTION HasCh(Ch : CHAR; VAR Pos : WORD) : BOOLEAN;
         FUNCTION IsChar(W : WORD; Ch : CHAR) : BOOLEAN;
         FUNCTION LastCharIs(Ch : CHAR) : BOOLEAN;
         FUNCTION LastNonSpaceChar(VAR Ch : CHAR) : WORD;
         PROCEDURE RemoveLastChar;
         PROCEDURE SetChar(W : WORD; Ch : CHAR);
         { *** WITH OTHER DATA TYPES *** }
         FUNCTION AppendBoolean(B : BOOLEAN; bt : INTEGER) : PChar;
         FUNCTION AppendByte(B : BYTE) : PChar;
         FUNCTION AppendChar(C : CHAR) : PChar;
         FUNCTION AppendExt(E : EXTENDED; Width,Places : BYTE) : PChar;
         FUNCTION AppendDouble(Mask: PChar; X : DOUBLE) : PChar;
         FUNCTION AppendLong(L : LONGINT) : PChar;
         FUNCTION AppendReal(R : REAL; Width,Places : BYTE) : PChar;
         FUNCTION FromBoolean(B : BOOLEAN; bt : INTEGER) : PChar;
         FUNCTION FromByte(B : BYTE) : PChar;
         FUNCTION FromChar(C : CHAR) : PChar;
         FUNCTION FromDouble(Mask: PChar; X : DOUBLE) : PChar;
         FUNCTION FromExt(E : EXTENDED; Width,Places : BYTE) : PChar;
         FUNCTION FromLong(L : LONGINT) : PChar;
         FUNCTION FromPtr(P : POINTER) : PChar;
         FUNCTION FromReal(R : REAL; Width,Places : BYTE) : PChar;
         FUNCTION FromRGB(C : TColorRef) : PChar;
         FUNCTION HexFromByte(B : BYTE) : PChar;
         FUNCTION HexFromLong(L : LONGINT) : PChar;
         FUNCTION HexFromPtr(P : POINTER) : PChar;
         FUNCTION HexFromWord(W : WORD) : PChar;
         FUNCTION ToBoolean(VAR B : BOOLEAN) : BOOLEAN;
         FUNCTION ToByte(VAR B : BYTE) : BOOLEAN;
         FUNCTION ToChar(VAR C : CHAR) : BOOLEAN;
         FUNCTION ToDouble(VAR D : DOUBLE) : BOOLEAN;
         FUNCTION ToExt(VAR E : EXTENDED) : BOOLEAN;
         FUNCTION ToLong(VAR L : LONGINT) : BOOLEAN;
         FUNCTION ToReal(VAR R : REAL) : BOOLEAN;
         FUNCTION ToRGB(VAR C : TColorRef) : BOOLEAN;
         { *** equivalent PChar type 'STRINGS' unit functions *** }
         FUNCTION StrCat(Source : PChar) : PChar;
         FUNCTION StrComp(Str2 : PChar) : INTEGER;
         FUNCTION StrCopy(Source : PChar) : PChar;
         FUNCTION StrECopy(Source : PChar) : PChar;
         FUNCTION StrEnd : PChar;
         FUNCTION StrIComp(Str2 : PChar) : INTEGER;
         FUNCTION StrLCat(Source : PChar; MaxLen : WORD) : PChar;
         FUNCTION StrLIComp(Str2 : PChar; MaxLen : WORD) : INTEGER;
         FUNCTION StrLComp(Str2 : PChar; MaxLen : WORD) : INTEGER;
         FUNCTION StrLCopy(Str2 : PChar; MaxLen : WORD) : INTEGER;
         FUNCTION StrLen : WORD;
         FUNCTION StrLower : PChar;
         FUNCTION StrMove(Source : PChar; Count : WORD) : PChar;
         FUNCTION StrPas : STRING;
         FUNCTION StrPCopy(Source : STRING) : PChar;
         FUNCTION StrPos(Str2 : PChar) : PChar;
         FUNCTION StrRScan(Chr : CHAR) : PChar;
         FUNCTION StrScan(Chr : CHAR) : PChar;
         FUNCTION StrUpper : PChar;
         { *** RESOURCE STRING LOADING *** }
         FUNCTION LoadString(Instance : THandle; Id : WORD) : PChar;
         { *** INI file related *** }
         FUNCTION ReadIniKeyword(AnIniFile,ASection,AKeyWord : PChar) : WORD;
         FUNCTION WriteIniKeyword(AnIniFile,ASection,AKeyWord : PChar) : WORD;
         { *** TRIM related *** }
         FUNCTION Delete(Index,Count : WORD) : PChar;
         FUNCTION Insert(NewStr : PChar; Index : WORD) : PChar;
         FUNCTION InsertL(NewStr : PChar; Len,Index : WORD) : PChar;
         FUNCTION PadCentre(NewLen : WORD; Ch : CHAR) : PChar;
         FUNCTION PadEnd(NewLen : WORD; Ch : CHAR) : PChar;
         FUNCTION PadFront(NewLen : WORD; Ch : CHAR) : PChar;
         FUNCTION RemoveDIC : PChar;
         FUNCTION RemoveSIC : PChar;
         FUNCTION Trim : PChar;
         FUNCTION TrimEnd : PChar;
         FUNCTION TrimFront : PChar;
         FUNCTION TrimZero : PChar;
         { *** COMPARISON METHODS *** }
         FUNCTION Compare(Other : PChar) : INTEGER;
         FUNCTION CompareI(Other : PChar) : INTEGER;
         FUNCTION CompareL(Other : PChar; Len : WORD) : INTEGER;
         FUNCTION CompareLI(Other : PChar; Len : WORD) : INTEGER;
         FUNCTION CompareLong(L : LONGINT) : INTEGER;
         FUNCTION CompareNum(D : DOUBLE) : INTEGER;
         FUNCTION IsSame(Other : PChar) : BOOLEAN;
         FUNCTION IsSameI(Other : PChar) : BOOLEAN;
         FUNCTION IsSameL(Other : PChar; Len : WORD) : BOOLEAN;
         FUNCTION IsSameLI(Other : PChar; Len : WORD) : BOOLEAN;
         { *** DOS PATH/FILE NAME RELATED *** }
         FUNCTION AddBackSlash : PChar;
         FUNCTION BuildPathName(ADirectory,AFileName,AnExt : PChar) : PChar;
         FUNCTION DefaultExtension(Ext : PChar) : PChar;
         FUNCTION ForceExtension(Ext : PChar) : PChar;
         FUNCTION HasBackSlash : BOOLEAN;
         FUNCTION HasDrive : BOOLEAN;
         FUNCTION HasExtension(VAR DotPos : WORD) : BOOLEAN;
         FUNCTION HasFileName : BOOLEAN;
         FUNCTION HasDirectory : BOOLEAN;
         FUNCTION DirectoryExists : BOOLEAN;
         FUNCTION DriveExists : BOOLEAN;
         FUNCTION FileExists : BOOLEAN;
         FUNCTION JustExtension(PathName : PChar) : PChar;
         FUNCTION JustFileName(PathName : PChar) : PChar;
         FUNCTION JustDirectory(PathName : PChar) : PChar;
         FUNCTION SetCurDir : BOOLEAN;
         { *** SEARCH RELATED FUNCTIONS *** }
         FUNCTION ChCount(Ch : CHAR) : WORD;
	      FUNCTION FindFirst(SubStr : PChar; VAR P : WORD) : BOOLEAN;
	      FUNCTION FindFirstCh(Ch : CHAR; VAR P : WORD) : BOOLEAN;
	      FUNCTION FindLast(SubStr : PChar; VAR P : WORD) : BOOLEAN;
	      FUNCTION FindLastCh(Ch : CHAR; VAR P : WORD) : BOOLEAN;
	      FUNCTION FindNext(SubStr : PChar; StartPos : WORD; VAR NextPos : WORD) : BOOLEAN;
	      FUNCTION FindNextCh(Ch : CHAR; StartPos : WORD; VAR NextPos : WORD) : BOOLEAN;
	      FUNCTION FindPrev(SubStr : PChar; StartPos : WORD; VAR PrevPos : WORD) : BOOLEAN;
	      FUNCTION FindPrevCh(Ch : CHAR; StartPos : WORD; VAR PrevPos : WORD) : BOOLEAN;
         FUNCTION SubStrCount(SubStr : PChar) : WORD;
         { *** Case related *** }
         FUNCTION FirstCharToUpper : PChar;
         FUNCTION ToLower : PChar;
         FUNCTION ToUpper : PChar;
         { *** Search & replace related methods *** }
         FUNCTION ReplaceAll(OldStr,NewStr : PChar) : PChar;
         FUNCTION ReplaceFirst(OldStr,NewStr : PChar) : PChar;
         FUNCTION ReplaceLast(OldStr,NewStr : PChar) : PChar;
         FUNCTION ReplaceChAll(OldCh,NewCh : CHAR) : PChar;
         FUNCTION ReplaceChFirst(OldCh,NewCh : CHAR) : PChar;
         FUNCTION ReplaceChLast(OldCh,NewCh : CHAR) : PChar;
         { *** PARSER RELATED *** }
         FUNCTION FirstParseDelim(Source       : PChar;
                                  DelimCh      : CHAR;
                                  VAR DelimPos : WORD) : BOOLEAN;
         FUNCTION NextParseDelim(Source           : PChar;
                                 DelimCh          : CHAR;
                                 StartPos         : WORD;
                                 VAR NextDelimPos : WORD) : BOOLEAN;
         FUNCTION ParseDelimCount(DelimCh : CHAR) : WORD;
         FUNCTION ParseDelimToList(DelimCh   : CHAR;
                                   Special   : INTEGER;
                                   VAR AList : PCollection) : WORD;
         FUNCTION ParsePosToList(VAR PosArray; PosCt : WORD; VAR AList : PCollection) : WORD;
   END;

   { ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

{$IFDEF UseStreams}

CONST

   RStringObject : TStreamRec = (
      ObjType : IStringObject;
      VmtLink : OFS(TYPEOF(TStringObject)^);
      Load    : @TStringObject.Load;
      Store   : @TStringObject.Store);

{$ENDIF}

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

IMPLEMENTATION

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

USES

   WinDos,
   Strings,
   SafeStr;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

CONST

   NewLineStr : ARRAY[0..2] OF CHAR = #13#10#0;

   TempStrLen  = 255;
   TempLStrLen = 20000;

   HStr : ARRAY[0..15] OF CHAR = '0123456789ABCDEF';

   BStr : ARRAY[bt_YesNo..bt_OnOff,BOOLEAN] OF PChar = (('No','Yes'),
                                                        ('0','1'),
                                                        ('False','True'),
                                                        ('F','T'),
                                                        ('N','Y'),
                                                        ('Off','On'));
VAR

   sThousand   : CHAR;          { thousand separator }
   sDecimal    : CHAR;          { Decimal character }
   sCurrency   : CHAR;          { Currency character }
   iCurrDigits : INTEGER;       { default number of decimal places }

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

CONSTRUCTOR TStringObject.InitDefault;
BEGIN
   INHERITED Init;
   InitDataMembers;
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitSize(AMaxSize: WORD; AStr: PChar);
VAR
   L : WORD;
BEGIN
   INHERITED Init;
   InitDataMembers;
   IF NOT ExpandBy(AMaxSize) THEN
      Fail
   ELSE
      Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.Init(AStr: PChar);
VAR
   L : WORD;
BEGIN
   INHERITED Init;
   InitDataMembers;
   Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitNL(AStr: PChar);
BEGIN
   INHERITED Init;
   InitDataMembers;
   Assign(AStr);
   Append(NewLineStr);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitString(S : STRING);
BEGIN
   INHERITED Init;
   InitDataMembers;
   AssignString(S);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitBoolean(B : BOOLEAN; StrType : INTEGER);
BEGIN
   INHERITED Init;
   InitDataMembers;
   FromBoolean(B,StrType);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitExt(E : EXTENDED; Width,Places : BYTE);
BEGIN
   INHERITED Init;
   InitDataMembers;
   FromExt(E,Width,Places);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitLong(L : LONGINT);
BEGIN
   INHERITED Init;
   InitDataMembers;
   FromLong(L);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitPtr(P : POINTER);
BEGIN
   INHERITED Init;
   InitDataMembers;
   FromPtr(P);
END;

{ ---------------------------------------------------------------------- }

CONSTRUCTOR TStringObject.InitReal(R : REAL; Width,Places : BYTE);
BEGIN
   INHERITED Init;
   InitDataMembers;
   FromReal(R,Width,Places);
END;

{ ---------------------------------------------------------------------- }

DESTRUCTOR TStringObject.Done;
BEGIN
   DisposeStr;
   INHERITED Done;
END;

{ ---------------------------------------------------------------------- }

{$IFDEF UseStreams}

CONSTRUCTOR TStringObject.Load(VAR S : TStream);
BEGIN
   InitDataMembers;
   TheStr := S.StrRead;
   IF TheStr <> NIL THEN
   BEGIN
      TheLength := Strings.StrLen(TheStr);
      TheMaxSize := TheLength;
   END;
END;

{$ENDIF}

{ ---------------------------------------------------------------------- }

{$IFDEF UseStreams}

PROCEDURE TStringObject.Store(VAR S : TStream);
BEGIN
   S.StrWrite(TheStr);
END;

{$ENDIF}

{ ---------------------------------------------------------------------- }


FUNCTION TStringObject.GetLength : WORD;
BEGIN
   GetLength := TheLength;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.GetPChar : PChar;
BEGIN
   GetPChar := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.GetString : STRING;
BEGIN
   GetString := Strings.StrPas(TheStr);
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.SetSizeInc(ASize : WORD);
BEGIN
   TheSizeInc := ASize;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Append(NewStr : PChar) : PChar;
VAR
   Extra : WORD;
BEGIN
   Append := TheStr;
   IF (NewStr <> NIL) AND (NewStr[0] <> #0) THEN
   BEGIN
      IF NOT CanCat(NewStr,Extra) THEN
      BEGIN
         IF NOT ExpandBy(Extra) THEN
            EXIT;
      END;
      Strings.StrCat(TheStr,NewStr);
      TheLength := SafeStrLen(TheStr);
   END;
   Append := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Append2(Part1,Part2 : PChar) : PChar;
BEGIN
   Append(Part1);
   Append2 := Append(Part2);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendLen(Source : PChar; Len : WORD) : PChar;
VAR
   G : WORD;
   C : CHAR;
BEGIN
   G := SafeStrLen(Source);
   IF G = 0 THEN
      AppendLen := TheStr
   ELSE
   BEGIN
      IF G <= Len THEN
         AppendLen := Append(Source)
      ELSE
      BEGIN
         C := Source[Len];
         Source[Len] := #0;
         AppendLen := Append(Source);
         Source[Len] := C;
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendMid(Source : PChar; Start,Count : WORD) : PChar;
VAR
   L : WORD;
BEGIN
   L := SafeStrLen(Source);
   IF (L = 0) OR (Start >= L) THEN
      AppendMid := TheStr
   ELSE
   BEGIN
      IF (Start+Count) >= L THEN
         AppendMid := Append(@Source[Start])
      ELSE
         AppendMid := AppendLen(@Source[Start],Count)
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendNL(NewStr : PChar) : PChar;
BEGIN
   Append(NewStr);
   AppendNL := Append(NewLineStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendPad(Source : PChar; Len : WORD; Ch : CHAR) : PChar;
BEGIN
   Append(Source);
   AppendPad := PadEnd(Len,Ch);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendTrim(Source : PChar) : PChar;
VAR
   TempObj : PStringObject;
BEGIN
   TempObj := NEW(PStringObject,Init(Source));
   IF TempObj^.GetLength > 0 THEN
   BEGIN
      TempObj^.Trim;
      AppendO(TempObj);
   END;
   DISPOSE(TempObj,Done);
   AppendTrim := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Assign(NewStr : PChar) : PChar;
VAR
   Extra : WORD;
BEGIN
   Assign := TheStr;
   IF NOT CanCopy(NewStr,Extra) THEN
   BEGIN
      IF NOT ExpandBy(Extra) THEN
         EXIT;
   END;
   Assign := SafeStrCopy(TheStr,NewStr);
   TheLength := SafeStrLen(TheStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignLen(NewStr : PChar; Len : WORD) : PChar;
VAR
   G : WORD;
   C : CHAR;
BEGIN
   G := SafeStrLen(NewStr);
   IF G <= Len THEN
      AssignLen := Assign(NewStr)
   ELSE
   BEGIN
      C := NewStr[Len];
      NewStr[Len] := #0;
      AssignLen := Assign(NewStr);
      NewStr[Len] := C;
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignMid(Source : PChar; Start,Count : WORD) : PChar;
VAR
   L : WORD;
BEGIN
   L := SafeStrLen(Source);
   IF (L = 0) OR (Start >= L) THEN
      Assign(NIL)
   ELSE
   BEGIN
      IF Start+Count >= L THEN
         AssignMid := Assign(@Source[Start])
      ELSE
         AssignMid := AssignLen(@Source[Start],Count);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignNL(NewStr : PChar) : PChar;
BEGIN
   Assign(NewStr);
   AssignNL := Append(NewLineStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignPad(Source : PChar; Len : WORD; Ch : CHAR) : PChar;
BEGIN
   Assign(Source);
   PadEnd(Len,Ch);
   AssignPad := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignAppend(Source,Cat : PChar) : PChar;
BEGIN
   Assign(Source);
   AssignAppend := Append(Cat);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignAppend2(Source,Cat1,Cat2 : PChar) : PChar;
BEGIN
   Assign(Source);
   Append(Cat1);
   AssignAppend2 := Append(Cat2);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendSIC(Source : PChar) : PChar;
CONST
   SIC = #39;
BEGIN
   AppendChar(SIC);
   Append(Source);
   AppendSIC := AppendChar(SIC);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendDIC(Source : PChar) : PChar;
CONST
   DIC = #34;
BEGIN
   AppendChar(DIC);
   Append(Source);
   AppendDIC := AppendChar(DIC);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignTrim(Source : PChar) : PChar;
VAR
   L,S,E : WORD;
BEGIN
   L := SafeStrLen(Source);
   IF L = 0 THEN
      AssignTrim := Assign(Source)
   ELSE
   BEGIN
      s := 0;
      WHILE Source[s] = ' ' DO
         INC(s);
      IF Source[s] = #0 THEN
      BEGIN
         DisposeStr;
         AssignTrim := TheStr;
      END
      ELSE
      BEGIN
         e := L-1;
         WHILE (e > s) AND (Source[e] = ' ') DO
            DEC(e);
         AssignTrim := AssignLen(@Source[s],e-s+1)
      END
   END;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Prepend(NewStr : PChar) : PChar;
VAR
   L : WORD;
   AStr : PChar;
BEGIN
   IF TheLength = 0 THEN
      Prepend := Assign(NewStr)
   ELSE
   BEGIN
      L := TheLength+1;
      GetMem(AStr,L);
      Strings.StrCopy(AStr,TheStr);
      Assign(NewStr);
      Append(AStr);
      FreeMem(AStr,L);
      Prepend := TheStr;
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendO(Other : PStringObject) : PChar;
BEGIN
   AppendO := Append(Other^.GetPChar);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AssignO(Other : PStringObject) : PChar;
BEGIN
   AssignO := Assign(Other^.GetPChar);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareO(Other       : PStringObject;
                                IgnoreCase  : BOOLEAN;
                                Len : WORD) : INTEGER;
BEGIN
   IF IgnoreCase THEN
   BEGIN
      IF Len > 0 THEN
         CompareO := StrLIComp(Other^.GetPChar,Len)
      ELSE
         CompareO := StrIComp(Other^.GetPChar)
   END
   ELSE
   BEGIN
      IF Len > 0 THEN
         CompareO := StrLComp(Other^.GetPChar,Len)
      ELSE
         CompareO := StrComp(Other^.GetPChar)
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsSameO(Other : PStringObject) : BOOLEAN;
BEGIN
   IsSameO := (CompareO(Other,FALSE,0) = 0);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IncludesO(Other : PStringObject) : BOOLEAN;
BEGIN
   IncludesO := (StrPos(Other^.GetPChar) <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.PrependO(Other : PStringObject) : PChar;
BEGIN
   PrependO := Prepend(Other^.GetPChar);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.WithinO(Other : PStringObject) : BOOLEAN;
BEGIN
   IF Other = NIL THEN
      WithinO := FALSE
   ELSE
      WithinO := (Other^.StrPos(TheStr) <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrCat(Source : PChar) : PChar;
BEGIN
   StrCat := Append(Source);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrComp(Str2 : PChar) : INTEGER;
BEGIN
   StrComp := Compare(Str2);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrCopy(Source : PChar) : PChar;
BEGIN
   StrCopy := Assign(Source);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrECopy(Source : PChar) : PChar;
BEGIN
   Assign(Source);
   IF TheStr = NIL THEN
      StrECopy := NIL
   ELSE
      StrECopy := Strings.StrEnd(TheStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrEnd : PChar;
BEGIN
   IF TheLength = 0 THEN
      StrEnd := NIL
   ELSE
      StrEnd := @TheStr[TheLength]
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrIComp(Str2 : PChar) : INTEGER;
BEGIN
   StrIComp := CompareI(Str2);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLCat(Source : PChar; MaxLen : WORD) : PChar;
BEGIN
   StrLCat := AppendLen(Source,MaxLen);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLComp(Str2 : PChar; MaxLen : WORD) : INTEGER;
BEGIN
   StrLComp := CompareL(Str2,MaxLen);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLCopy(Str2 : PChar; MaxLen : WORD) : INTEGER;
VAR
   L : WORD;
BEGIN
   AssignLen(Str2,MaxLen);
   L := SafeStrLen(Str2);
   IF L > MaxLen THEN
      StrLCopy := MaxLen
   ELSE
      StrLCopy := L
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLen : WORD;
BEGIN
   StrLen := GetLength;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLIComp(Str2 : PChar; MaxLen : WORD) : INTEGER;
BEGIN
   StrLIComp := CompareLI(Str2,MaxLen);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrLower : PChar;
BEGIN
   StrLower := ToLower
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrMove(Source : PChar; Count : WORD) : PChar;
VAR
   Extra : WORD;
   L,E : INTEGER;
BEGIN
   StrMove := TheStr;
   IF SafeStrLen(Source) > 0 THEN
   BEGIN
      IF NOT CanCopy(Source,Extra) THEN
      BEGIN
         IF NOT ExpandBy(Extra) THEN
            EXIT;
      END;
      Strings.StrMove(TheStr,Source,Count);
   END;
   StrMove := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrPas : STRING;
BEGIN
   IF (TheStr = NIL) OR (TheLength > 255) THEN
      StrPas := ''
   ELSE
      StrPas := Strings.StrPas(TheStr)
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrPCopy(Source : STRING) : PChar;
VAR
   L : INTEGER;
   AStr : PChar;
BEGIN
   L := System.LENGTH(Source);
   IF L = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      GetMem(AStr,L+1);
      Strings.StrPCopy(AStr,Source);
      Assign(AStr);
      FreeMem(AStr,L+1);
   END;
   StrPCopy := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrPos(Str2 : PChar) : PChar;
BEGIN
   IF (TheLength = 0) OR (SafeStrLen(Str2) = 0) THEN
      StrPos := NIL
   ELSE
      StrPos := Strings.StrPos(TheStr,Str2);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrRScan(Chr : CHAR) : PChar;
BEGIN
   IF TheStr = NIL THEN
      StrRScan := NIL
   ELSE
      StrRScan := Strings.StrRScan(TheStr,Chr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrScan(Chr : CHAR) : PChar;
BEGIN
   IF TheStr = NIL THEN
      StrScan := NIL
   ELSE
      StrScan := Strings.StrScan(TheStr,Chr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.StrUpper : PChar;
BEGIN
   StrUpper := ToUpper;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendBoolean(B : BOOLEAN; bt : INTEGER) : PChar;
VAR
   TempObj : PStringObject;
BEGIN
   TempObj := NEW(PStringObject,InitBoolean(B,bt));
   AppendBoolean := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendByte(B : BYTE) : PChar;
VAR
   AStr : ARRAY[0..10] OF CHAR;
BEGIN
   STR(B,AStr);
   AppendByte := Append(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendChar(C : CHAR) : PChar;
VAR
   AStr : ARRAY[0..1] OF CHAR;
BEGIN
   AStr[0] := C;
   AStr[1] := #0;
   AppendChar := Append(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendExt(E : EXTENDED; Width,Places : BYTE) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(E:Width:Places,AStr);
   AppendExt := Append(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendDouble(Mask: PChar; X : DOUBLE) : PChar;
VAR
   TempObj : PStringObject;
BEGIN
   TempObj := NEW(PStringObject,InitDefault);
   TempObj^.FromDouble(Mask,X);
   AppendDouble := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendLong(L : LONGINT) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(L,AStr);
   AppendLong := Append(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AppendReal(R : REAL; Width,Places : BYTE) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(R:Width:Places,AStr);
   AppendReal := Append(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromBoolean(B : BOOLEAN; bt : INTEGER) : PChar;
BEGIN
   IF (bt >= 1) AND (bt <= bt_Last) THEN
      FromBoolean := Assign(BStr[bt,B]);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromByte(B : BYTE) : PChar;
VAR
   AStr : ARRAY[0..10] OF CHAR;
BEGIN
   STR(B,AStr);
   FromByte := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromChar(C : CHAR) : PChar;
VAR
   AStr : ARRAY[0..1] OF CHAR;
BEGIN
   AStr[0] := C;
   AStr[1] := #0;
   FromChar := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromExt(E : EXTENDED; Width,Places : BYTE) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(E:Width:Places,AStr);
   FromExt := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromLong(L : LONGINT) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(L,AStr);
   FromLong := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromPtr(P : POINTER) : PChar;
VAR
   TempObj : PStringObject;
   w : WORD;
BEGIN
   W := SEG(P);
   HexFromWord(W);
   Append(':');
   TempObj := NEW(PStringObject,InitDefault);
   W := OFS(P);
   TempObj^.HexFromWord(W);
   FromPtr := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromReal(R : REAL; Width,Places : BYTE) : PChar;
VAR
   AStr : ARRAY[0..256] OF CHAR;
BEGIN
   STR(R:Width:Places,AStr);
   FromReal := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToBoolean(VAR B : BOOLEAN) : BOOLEAN;
VAR
   F : BOOLEAN;
   bt : INTEGER;
BEGIN
   F := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      bt := 1;
      WHILE (NOT F) AND (bt <= bt_Last) DO
      BEGIN
         IF SafeStrIComp(TheStr,BStr[bt,FALSE]) = compare_Eq THEN
         BEGIN
            F := TRUE;
            B := FALSE
         END
         ELSE
         BEGIN
            IF SafeStrIComp(TheStr,BStr[bt,TRUE]) = compare_Eq THEN
            BEGIN
               F := TRUE;
               B := TRUE
            END
            ELSE
               INC(bt);
         END
      END
   END;
   ToBoolean := F;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToByte(VAR B : BYTE) : BOOLEAN;
VAR
   C : INTEGER;
BEGIN
   ToByte := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      VAL(TheStr,B,C);
      IF C = 0 THEN
         ToByte := TRUE
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToChar(VAR C : CHAR) : BOOLEAN;
BEGIN
   IF TheLength > 0 THEN
   BEGIN
      C := TheStr[0];
      ToChar := TRUE;
   END
   ELSE
      ToChar := FALSE;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToDouble(VAR D : DOUBLE) : BOOLEAN;
VAR
   C : INTEGER;
BEGIN
   ToDouble := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      VAL(TheStr,D,C);
      ToDouble := (C = 0);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToExt(VAR E : EXTENDED) : BOOLEAN;
VAR
   C : INTEGER;
BEGIN
   ToExt := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      VAL(TheStr,E,C);
      ToExt := (C = 0);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToLong(VAR L : LONGINT) : BOOLEAN;
VAR
   C : INTEGER;
BEGIN
   ToLong := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      VAL(TheStr,L,C);
      ToLong := (C = 0);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToReal(VAR R : REAL) : BOOLEAN;
VAR
   C : INTEGER;
BEGIN
   ToReal := FALSE;
   IF TheLength > 0 THEN
   BEGIN
      VAL(TheStr,R,C);
      ToReal := (C = 0);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToRGB(VAR C : TColorRef) : BOOLEAN;
VAR
   AList : PCollection;
   R,G,B : BYTE;
BEGIN
   ToRGB := FALSE;
   { parse 'xx,xx,xx' into three string objects }
   AList := NEW(PCollection,Init(3,3));
   ParseDelimToList(',',delim_None,AList);
   IF AList^.Count = 3 THEN
   BEGIN
      IF PStringObject(AList^.At(0))^.ToByte(R)
      AND PStringObject(AList^.At(1))^.ToByte(G)
      AND PStringObject(AList^.At(2))^.ToByte(B) THEN
      BEGIN
         C := RGB(R,G,B);
         ToRGB := TRUE;
      END
   END;
   DISPOSE(AList,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HexFromByte(B : BYTE) : PChar;
VAR
   AStr : ARRAY[0..2] OF CHAR;
BEGIN
   AStr[0] := HStr[B SHR 4];
   AStr[1] := HStr[B AND $F];
   AStr[2] := #0;
   HexFromByte := Assign(AStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromDouble(Mask: PChar; X : DOUBLE) : PChar;
CONST
   SPACE   = ' ';
   ZERO    = '0';
   RIGHT   = 1;
   LEFT    = 2;
   InputChars : PChar = '+-#@*';
TYPE
   SignLogic = (Default,Plus,Minus);
VAR
   RetStr    : ARRAY[0..80] OF CHAR;
   FillCh    : CHAR;
   DecCh     : CHAR;
   SepCh     : CHAR;
   SignCh    : CHAR;
   I	     : INTEGER;
   J	     : BYTE;
   IntLen    : BYTE;
   DecPos    : INTEGER;
   Start     : INTEGER;
   EndIt     : BYTE;
   NFldSize  : BYTE;
   DPlaces   : BYTE;
   SignPos   : BYTE;
   SignFlg   : SignLogic;
   Finished  : BOOLEAN;
   Money     : BOOLEAN;
   InNum     : BOOLEAN;
   Negative  : BOOLEAN;
   Dest  : PChar;

BEGIN

   GetMem(Dest,TempStrLen);
   Strings.StrCopy(Dest,#0);

   I := SafeStrLen(Mask);
   IF (I > 0) AND (I < SIZEOF(RetStr)) THEN
   BEGIN
      Finished   := FALSE;
      InNum      := FALSE;
      Money      := FALSE;
      SignFlg    := Default;
      Negative   := X < 0;
      DecPos     := -1;
      I	         := -1;
      Start      := -1;
      FillCh     := SPACE;
      SepCh      := #0;
      DecCh      := #0;
      X	         := Abs(X); { Remove sign--replace later }

      { add currency symbol to inputChars list }
      InputChars[5] := sCurrency;
      InputChars[6] := #0;

      { Extract the numeric sequence from the mask.  This field determines}
      {	where the formatted value is going to go.			}
      { Determine whether a floating money sign is requested.		}
      { Get the fill character (' ','*','0')                              }
      { Get decimal character and thousand's separator character.         }
      { Get sign position and whether '+' is used FOR positive numbers.   }
      { No ',', '.' ' ' outside of OR at either END of numeric field.     }

      repeat
         Inc(I);
         IF ((InNum) and ((Mask[I] = sDecimal)
         OR (Mask[I] = sThousand) OR (Mask[I] = SPACE)))
         OR (Strings.StrScan(InputChars,Mask[I]) <> NIL) THEN
         BEGIN
            InNum := TRUE;
	         IF (Start = -1) THEN
   	         Start := I;
	         IF (Mask[I] = '-') THEN
	         BEGIN
	            SignPos := I;
	            SignFlg := Minus
	         END;
	         IF (Mask[I] = '+') THEN
	         BEGIN
	            SignPos := I;
	            SignFlg := Plus
	         END;
	         IF Mask[I] = sCurrency THEN
	            Money := TRUE;
	         IF Mask[I] = '@' THEN
                 BEGIN
	            IF FillCh = SPACE THEN
	              FillCh := ZERO;
                 END;
	         IF (Mask[I] = '*') THEN
                    FillCh := '*';
	         IF ((Mask[I] = sDecimal)
            OR (Mask[I] = sThousand) OR (Mask[I] = SPACE)) THEN
            BEGIN
               IF (I = (SafeStrLen(Mask)-1))
               OR (SafeChPos(InputChars,Mask[Succ(I)]) < 2) THEN
                  Finished := TRUE
               ELSE
               BEGIN
   	            IF (Mask[I] = SPACE) THEN
	                  SepCh := SPACE
	               ELSE
	               BEGIN
	                  IF (DecCh <> #0) THEN
		                  SepCh  := DecCh;
		               DecCh  := Mask[I];
		               DecPos := I
                  END
               END
	         END
         END
         ELSE
         BEGIN
   	      IF (InNum) THEN	       { Found the END of it. }
	            Finished := TRUE;
         END
      UNTIL (I = (SafeStrLen(Mask)-1)) OR Finished;

      IF Start < 0 THEN       { No numeric field; return Mask. }
        Strings.StrCopy(Dest,Mask)
      ELSE
      BEGIN
         IF (SignFlg <> Default) THEN
         BEGIN
            IF (SignPos = I) THEN  { Sign at left unless at rt.end }
            BEGIN
               Inc(DecPos, Ord(DecPos > 0));
	       SignPos := RIGHT
            END
            ELSE
               SignPos := LEFT;
         END;
         IF (Money) THEN   { No '0' ahead of '$' }
         BEGIN
            IF (FillCh = ZERO) THEN
               FillCh := SPACE;
         END;
         EndIt := I - Ord(Finished);     { Trailing characters.	}

         { Now Start and EndIt contain beginning/ending position of numeric	}
         { field.  Extract numeric sequence from Mask.			}

         NFldSize := Succ(EndIt - Start);

         { Adjust DecPos to its position within
           the numeric sequence, THEN get decimal places. }

         IF (DecPos <> -1) THEN
         BEGIN
            DecPos  := DecPos - Pred(Start);
            DPlaces := NFldSize - DecPos
         END
         ELSE
            DPlaces := 0;

         { Make raw numeric string and insert the proper decimal character,	}
         { THEN insert separators.						}

         Str(X : 0 : DPlaces, RetStr);

         IF (DPlaces > 0) THEN
            RetStr[Strings.StrLen(RetStr) - DPlaces - 1] := DecCh;

         J := 0;
         IF (DPlaces > 0) THEN
            IntLen := Strings.StrLen(RetStr) - Succ(DPlaces)
         ELSE
            IntLen := Strings.StrLen(RetStr);
         IF (SepCh <> #0) THEN	{ Separators required.	}
         BEGIN
            FOR I := IntLen downto 2 do
            BEGIN
               Inc(J);
	       IF (J mod 3 = 0) THEN
                  SafeChInsert(RetStr,sThousand,I-1);
            END;
         END;

         IF (Negative) THEN
            SignCh := '-'
         ELSE
         BEGIN
            IF (SignFlg = Plus) THEN
	       SignCh := '+'
            ELSE
	       SignCh := SPACE;
         END;

         { Check to see IF numeric string is going to fit.			}
         { J is number of characters required FOR the formatted number.	}

         { Add money symbol, fill characters, and sign IF any.		}
         IF (Money) THEN
            SafeChInsert(RetStr,sCurrency,0);
         IF (SignFlg = Default) THEN
         BEGIN
            IF ((Negative) and (FillCh = SPACE)) THEN
               SafeChInsert(RetStr,'-',0);
            WHILE Strings.StrLen(RetStr) < NFldSize do
               SafeChInsert(RetStr,FillCh,0);
         END
         ELSE
         BEGIN
            CASE SignPos OF
	        RIGHT : BEGIN
                           RetStr[Strings.StrLen(RetStr)] := SignCh;
                           RetStr[Strings.StrLen(RetStr)+1] := #0;
                        END;
	        LEFT	 : SafeChInsert(RetStr,SignCh,0);
            END;
            WHILE Strings.StrLen(RetStr) < NFldSize DO
               SafeChInsert(RetStr,FillCh,SignPos);
         END;

         { Attach any literal characters from Mask.				}

         IF (Start > 0) THEN
         BEGIN
            Strings.StrLCat(Dest,Mask,Start);
            Strings.StrCat(Dest,RetStr);
         END
         ELSE
            Strings.StrCopy(Dest,RetStr);
         IF (EndIt < Strings.StrLen(Mask)-1) THEN
            SafeStrMid(Dest,Mask,EndIt+1,Strings.StrLen(Mask)-EndIt);

      END { Start = 0 }

   END;  { SafeStrLen(Mask) = 0 }

   FromDouble := Assign(Dest);

   FreeMem(Dest,TempStrLen);

END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HexFromPtr(P : POINTER) : PChar;
BEGIN
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FromRGB(C : TColorRef) : PChar;
BEGIN
   FromByte(GetRValue(C));
   Append(',');
   AppendByte(GetGValue(C));
   Append(',');
   FromRGB := AppendByte(GetBValue(C));
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HexFromWord(W : WORD) : PChar;
VAR
   TempObj : PStringObject;
   B : BYTE;
BEGIN
   B := HiByte(W);
   HexFromByte(B);
   B := LoByte(W);
   TempObj := NEW(PStringObject,InitDefault);
   TempObj^.HexFromByte(B);
   HexFromWord := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HexFromLong(L : LONGINT) : PChar;
VAR
   TempObj : PStringObject;
   W : WORD;
BEGIN
   W := HiWord(L);
   HexFromWord(W);
   TempObj := NEW(PStringObject,InitDefault);
   W := LoWord(L);
   TempObj^.HexFromWord(W);
   HexFromLong := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.LoadString(Instance : THandle; Id : WORD) : PChar;
VAR
   L : INTEGER;
   AStr : PChar;
BEGIN
   GetMem(AStr,255);
   L := WinProcs.LoadString(hInstance,Id,AStr,255);
   IF L <= 0 THEN
      DisposeStr
   ELSE
      Assign(AStr);
   FreeMem(AStr,255);
   LoadString := TheStr;
END;

{ ---------------------------------------------------------------------- }

{ Description : This presumes that the object 'TheStr' buffer holds
                the whole INI file section content, as read via
                the method 'ReadIniSection'. This process parses
                that buffer into two co-ordinated lists: a list of
                keywords, and a list of keyword values. }

FUNCTION TStringObject.ReadIniKeyword(AnIniFile,ASection,AKeyWord : PChar) : WORD;
VAR
   AStr,DefStr : PChar;
   R : WORD;
BEGIN
   GetMem(AStr,TempStrLen);
   GetMem(DefStr,TempStrLen);
   IF TheLength > 0 THEN
      Strings.StrCopy(DefStr,TheStr)
   ELSE
      Strings.StrCopy(DefStr,#0);
   R := GetPrivateProfileString(ASection,
                                AKeyword,
                                DefStr,
                                AStr,
                                TempStrLen,
                                AnIniFile);
   IF R > 0 THEN
      Assign(AStr);
   FreeMem(AStr,TempStrLen);
   FreeMem(DefStr,TempStrLen);
   ReadIniKeyword := R;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.WriteIniKeyword(AnIniFile,ASection,AKeyWord : PChar) : WORD;
BEGIN
   WritePrivateProfileString(ASection,AKeyword,TheStr,AnIniFile);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Delete(Index,Count : WORD) : PChar;
BEGIN
   IF TheLength > 0 THEN
   BEGIN
      SafeStrDelete(TheStr,Index,Count);
      TheLength := SafeStrLen(TheStr);
   END;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Insert(NewStr : PChar; Index : WORD) : PChar;
VAR
   L,Extra : WORD;
BEGIN
   Insert := TheStr;
   L := SafeStrLen(NewStr);
   IF L > 0 THEN
   BEGIN
      IF NOT CanCat(NewStr,Extra) THEN
      BEGIN
         IF NOT ExpandBy(Extra) THEN
            EXIT;
      END;
      SafeStrInsert(TheStr,NewStr,Index);
      TheLength := SafeStrLen(TheStr);
   END;
   Insert := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.InsertL(NewStr : PChar; Len,Index : WORD) : PChar;
VAR
   L : WORD;
   AStr : PChar;
BEGIN
   L := SafeStrLen(NewStr);
   IF L > 0 THEN
   BEGIN
      IF L <= Len THEN
         Insert(NewStr,Index)
      ELSE
      BEGIN
         GetMem(AStr,Len+1);
         Strings.StrLCopy(AStr,NewStr,Len);
         Insert(AStr,Index);
         FreeMem(AStr,Len+1);
      END
   END;
   InsertL := TheStr;
END;

{ ---------------------------------------------------------------------- }

{ Description : In a multi word string this function converts the
                first letter of each word to upper case. Example :
                'this is a test' ---> 'This Is A Test' }

FUNCTION TStringObject.FirstCharToUpper : PChar;
VAR
   E : INTEGER;
   MakeUpper : BOOLEAN;
BEGIN
   IF TheLength > 0 THEN
   BEGIN
      E := 0;
      MakeUpper := TRUE;
      WHILE TheStr[E] <> #0 DO
      BEGIN
         IF NOT MakeUpper THEN
         BEGIN
            IF TheStr[E] = ' ' THEN
               MakeUpper := TRUE;
         END
         ELSE
         BEGIN
            IF TheStr[E] <> ' ' THEN
            BEGIN
               TheStr[E] := UPCASE(TheStr[E]);
               MakeUpper := FALSE;
            END
         END;
         INC(E);
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToLower : PChar;
BEGIN
   IF TheLength > 0 THEN
      Strings.StrLower(TheStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ToUpper : PChar;
BEGIN
   IF TheLength > 0 THEN
      Strings.StrUpper(TheStr);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Compare(Other : PChar) : INTEGER;
BEGIN
   IF TheLength = 0 THEN
   BEGIN
      IF SafeStrlen(Other) = 0 THEN
         Compare := Compare_Eq
      ELSE
         Compare := Compare_LT
   END
   ELSE
   BEGIN
      IF SafeStrLen(Other) = 0 THEN
         Compare := Compare_GT
      ELSE
         Compare := CompConv(Strings.StrComp(TheStr,Other));
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareI(Other : PChar) : INTEGER;
BEGIN
   CompareI := CompConv(SafeStrIComp(TheStr,Other));
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareL(Other : PChar; Len : WORD) : INTEGER;
BEGIN
   IF TheLength = 0 THEN
   BEGIN
      IF SafeStrlen(Other) = 0 THEN
         CompareL := Compare_Eq
      ELSE
         CompareL := Compare_LT
   END
   ELSE
   BEGIN
      IF SafeStrLen(Other) = 0 THEN
         CompareL := Compare_GT
      ELSE
         CompareL := CompConv(Strings.StrLComp(TheStr,Other,Len));
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareLI(Other : PChar; Len : WORD) : INTEGER;
BEGIN
   IF TheLength = 0 THEN
   BEGIN
      IF SafeStrlen(Other) = 0 THEN
         CompareLI := Compare_Eq
      ELSE
         CompareLI := Compare_LT
   END
   ELSE
   BEGIN
      IF SafeStrLen(Other) = 0 THEN
         CompareLI := Compare_GT
      ELSE
         CompareLI := CompConv(Strings.StrLIComp(TheStr,Other,Len));
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareLong(L : LONGINT) : INTEGER;
VAR
   N : LONGINT;
BEGIN
   IF TheLength = 0 THEN
      CompareLong := compare_lt
   ELSE
   BEGIN
      IF NOT ToLong(N) THEN
         CompareLong := compare_lt
      ELSE
      BEGIN
         IF N > L THEN
            CompareLong := compare_gt
         ELSE
         BEGIN
            IF N < L THEN
               CompareLong := Compare_lt
            ELSE
               CompareLong := Compare_Eq
         END
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompareNum(D : DOUBLE) : INTEGER;
VAR
   N : DOUBLE;
BEGIN
   IF TheLength = 0 THEN
      CompareNum := compare_lt
   ELSE
   BEGIN
      IF NOT ToDouble(N) THEN
         CompareNum := compare_lt
      ELSE
      BEGIN
         IF N > D THEN
            CompareNum := compare_gt
         ELSE
         BEGIN
            IF N < D THEN
               CompareNum := Compare_lt
            ELSE
               CompareNum := Compare_Eq
         END
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsSame(Other : PChar) : BOOLEAN;
BEGIN
   IsSame := (Compare(Other) = compare_eq);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsSameI(Other : PChar) : BOOLEAN;
BEGIN
   IsSameI := (CompareI(Other) = compare_eq);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsSameL(Other : PChar; Len : WORD) : BOOLEAN;
BEGIN
   IsSameL := (CompareL(Other,Len) = compare_eq);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsSameLI(Other : PChar; Len : WORD) : BOOLEAN;
BEGIN
   IsSameLI := (CompareLI(Other,Len) = compare_eq);
END;

{ ---------------------------------------------------------------------- }

{ Description : Checks for and removes '"' double inverted commas
                from the start and end of the objects own data string. }

FUNCTION TStringObject.RemoveDIC : PChar;
CONST
   DIC = #34;
BEGIN
   IF FirstCharIs(DIC) AND LastCharIs(DIC) THEN
   BEGIN
      Delete(TheLength-1,1);
      Delete(0,1);
      TheLength := SafeStrLen(TheStr);
   END;
   RemoveDIC := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.RemoveSIC : PChar;
CONST
   SIC = #39;
BEGIN
   IF FirstCharIs(SIC) AND LastCharIs(SIC) THEN
   BEGIN
      Delete(TheLength-1,1);
      Delete(0,1);
      TheLength := SafeStrLen(TheStr);
   END;
   RemoveSIC := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.AddBackSlash : PChar;
VAR
   L : WORD;
BEGIN
   IF NOT LastCharIs('\') THEN
      Append('\');
   AddBackSlash := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.BuildPathName(ADirectory,AFileName,AnExt : PChar) : PChar;
VAR
   TempObj : PStringObject;
BEGIN
   Assign(ADirectory);
   AddBackSlash;
   TempObj := NEW(PStringObject,InitDefault);
   TempObj^.JustFileName(AFileName);
   AppendO(TempObj);
   TempObj^.Assign(AnExt);
   IF TempObj^.GetLength > 4 THEN
      TempObj^.JustExtension(AnExt);
   IF (TempObj^.GetLength > 0) AND (NOT TempObj^.FirstCharIs('.')) THEN
      TempObj^.Insert('.',1);
   BuildPathname := AppendO(TempObj);
   DISPOSE(TempObj,Done);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.DefaultExtension(Ext : PChar) : PChar;
VAR
   ADir : ARRAY[0..fsDirectory] OF CHAR;
   AName : ARRAY[0..fsFileName] OF CHAR;
   OldExt : ARRAY[0..fsExtension] OF CHAR;
   P : PChar;
BEGIN
   IF (TheLength > 0) AND (SafeStrLen(Ext) > 0) THEN
   BEGIN
      IF (FileSplit(TheStr,ADir,AName,OldExt) AND fcExtension) = 0 THEN
      BEGIN
         IF NOT LastCharIs('.') THEN
            Append('.');
         P := Strings.StrScan(Ext,'.');
         IF P <> NIL THEN
            P := @P[1]
         ELSE
            P := Ext;
         Append(P);
      END
   END;
   DefaultExtension := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ForceExtension(Ext : PChar) : PChar;
VAR
   ADir : ARRAY[0..fsDirectory] OF CHAR;
   AName : ARRAY[0..fsFileName] OF CHAR;
   AnExt : ARRAY[0..fsExtension] OF CHAR;
BEGIN
   IF (TheLength > 0) AND (SafeStrLen(Ext) > 0) THEN
   BEGIN
      FileSplit(TheStr,ADir,AName,AnExt);
      BuildPathName(ADir,AName,Ext);
   END;
   ForceExtension := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasBackSlash : BOOLEAN;
BEGIN
   HasBackSlash := LastCharIs('\');
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasDrive : BOOLEAN;
BEGIN
   IF (TheLength >= 3)
   AND (ORD(UpCase(TheStr[0])) >= 65)
   AND (ORD(UpCase(TheStr[0])) <= 90) THEN
      HasDrive := TRUE
   ELSE
      HasDrive := FALSE
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasExtension(VAR DotPos : WORD) : BOOLEAN;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
   H : BOOLEAN;
   P : PChar;
BEGIN
   H := FALSE;
   DotPos := WordResError;
   IF TheLength > 0 THEN
   BEGIN
      H := ((WinDos.FileSplit(TheStr,Dir,Name,Ext) AND fcExtension) = fcExtension);
      IF H THEN
      BEGIN
         P := Strings.StrRScan(TheStr,'.');
         IF P <> NIL THEN
            DotPos := P-TheStr;
      END
   END;
   HasExtension := H;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasFileName : BOOLEAN;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
BEGIN
   IF TheLength = 0 THEN
      HasFileName := FALSE
   ELSE
      HasFileName := ((WinDos.FileSplit(TheStr,Dir,Name,Ext) AND fcFileName) = fcFileName);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasDirectory : BOOLEAN;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
BEGIN
   IF TheLength = 0 THEN
      HasDirectory := FALSE
   ELSE
      HasDirectory := ((WinDos.FileSplit(TheStr,Dir,Name,Ext) AND fcDirectory) = fcDirectory);
END;

{ ---------------------------------------------------------------------- }

{ Description : Checks to see if the string is equivalent to an
                existing DOS directory. The process takes care
                of any trailing '\' characters. Returns TRUE if
                the DOS directory exists. }

FUNCTION TStringObject.DirectoryExists : BOOLEAN;
VAR
   TempObj : PStringObject;
   S : TSearchRec;
   D : BOOLEAN;
BEGIN
   D := FALSE;
   { ignore anything of 3 or less chars }
   IF TheLength > 3 THEN
   BEGIN
      { first of all try own string }
      WinDos.FindFirst(GetPChar,faDirectory,S);
      D := (DosError = 0);
      { if this fails then extract dir component from own string }
      IF NOT D THEN
      BEGIN
         TempObj := NEW(PStringObject,InitDefault);
         TempObj^.JustDirectory(GetPChar);
         { check for trailing '\' }
         IF TempObj^.LastCharIs('\') AND (TempObj^.GetLength > 3) THEN
            TempObj^.RemoveLastChar;
         WinDos.FindFirst(TempObj^.GetPChar,faDirectory,S);
         D := (DosError = 0);
         DISPOSE(TempObj,Done);
      END
   END;
   DirectoryExists := D;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.DriveExists : BOOLEAN;
VAR
   Ch : CHAR;
BEGIN
   IF TheLength = 0 THEN
      DriveExists := FALSE
   ELSE
   BEGIN
      Ch := UPCASE(TheStr[0]);
      DriveExists := ((Ch >= 'A') AND
                      (Ch <= 'Z') AND
                      (DiskFree(ORD(Ch)-64) <> -1));
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FileExists : BOOLEAN;
VAR
   S : TSearchRec;
BEGIN
   IF TheLength = 0 THEN
      FileExists := FALSE
   ELSE
   BEGIN
      WinDos.FindFirst(TheStr,faAnyFile,S);
      IF DosError <> 0 THEN
         FileExists := FALSE
      ELSE
         FileExists := ((S.Attr AND (faDirectory OR faVolumeID)) = 0);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.JustExtension(PathName : PChar) : PChar;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
   P : PChar;
BEGIN
   IF SafeStrLen(PathName) = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      FileSplit(PathName,Dir,Name,Ext);
      IF SafeStrLen(Ext) = 0 THEN
         DisposeStr
      ELSE
      BEGIN
         P := Strings.StrScan(Ext,'.');
         IF P <> NIL THEN
            P := @P[1]
         ELSE
            P := Ext;
         Assign(P);
      END
   END;
   JustExtension := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.JustFileName(PathName : PChar) : PChar;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
   P : PChar;
BEGIN
   IF SafeStrLen(PathName) = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      FileSplit(PathName,Dir,Name,Ext);
      IF SafeStrLen(Name) = 0 THEN
         DisposeStr
      ELSE
         Assign(Name);
   END;
   JustFileName := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.JustDirectory(PathName : PChar) : PChar;
VAR
   Dir : ARRAY[0..fsDirectory] OF CHAR;
   Name : ARRAY[0..fsFileName] OF CHAR;
   Ext : ARRAY[0..fsExtension] OF CHAR;
   P : PChar;
BEGIN
   IF SafeStrLen(PathName) = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      FileSplit(PathName,Dir,Name,Ext);
      IF SafeStrLen(Dir) = 0 THEN
         DisposeStr
      ELSE
         Assign(Dir);
   END;
   JustDirectory := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.SetCurDir : BOOLEAN;
VAR
   TempObj : PStringObject;
   T : BOOLEAN;

BEGIN
   T := FALSE;
   IF TheLength > 3 THEN
   BEGIN
      { first of all try it from the raw string }
      TempObj := NEW(PStringObject,Init(GetPChar));
      { check for trailing '\' }
      IF TempObj^.HasBackSlash THEN
         TempObj^.RemoveLastChar;
      WinDos.SetCurDir(TempObj^.GetPChar);
      T := (DosError = 0);
      { if that fails try it on the directory component }
      IF NOT T THEN
      BEGIN
         TempObj^.JustDirectory(GetPChar);
         { check for trailing '\' }
         IF TempObj^.LastCharIs('\') AND (TempObj^.GetLength > 3) THEN
            TempObj^.RemoveLastChar;
         WinDos.SetCurDir(TempObj^.GetPChar);
         T := (DosError = 0);
      END;
      DISPOSE(TempObj,Done);
   END;
   SetCurDir := T;
END;

{ ---------------------------------------------------------------------- }

{ Description : returns the number of times 'Ch' appears within the
                string object value. }

FUNCTION TStringObject.ChCount(Ch : CHAR) : WORD;
VAR
   E,C : WORD;
BEGIN
   IF TheLength = 0 THEN
      ChCount := 0
   ELSE
   BEGIN
      C := 0;
      E := 0;
      WHILE TheStr[E] <> #0 DO
      BEGIN
         IF TheStr[E] = Ch THEN
            INC(C);
         INC(E);
      END;
      ChCount := C;
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindFirst(SubStr : PChar; VAR P : WORD) : BOOLEAN;
VAR
   S : PChar;
BEGIN
   S := StrPos(SubStr);
   IF S <> NIL THEN
      P := S-TheStr;
   FindFirst := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindFirstCh(Ch : CHAR; VAR P : WORD) : BOOLEAN;
VAR
   S : PChar;
BEGIN
   S := StrScan(Ch);
   IF S <> NIL THEN
      P := S-TheStr;
   FindFirstCh := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindLast(SubStr : PChar; VAR P : WORD) : BOOLEAN;
VAR
   F : BOOLEAN;
   E : WORD;
   S : PChar;
BEGIN
   S := NIL;
   IF (SafeStrLen(SubStr) > 0) AND (TheLength > 0) THEN
   BEGIN
      E := TheLength-1;
      F := FALSE;
      WHILE (NOT F) AND (E >= 0) DO
      BEGIN
         IF SafeStrIComp(SubStr,@TheStr[E]) = 0 THEN
         BEGIN
            F := TRUE;
            S := @TheStr[E];
            P := S-TheStr;
         END
         ELSE
            DEC(E);
      END;
   END;
   FindLast := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindLastCh(Ch : CHAR; VAR P : WORD) : BOOLEAN;
VAR
   S : PChar;
BEGIN
   S := StrRScan(Ch);
   IF S <> NIL THEN
      P := S-TheStr;
   FindLastCh := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindNext(SubStr   : PChar;
                                StartPos : WORD;
                                VAR NextPos : WORD) : BOOLEAN;
VAR
   S,T : PChar;
BEGIN
   S := NIL;
   IF (SafeStrLen(SubStr) > 0)
   AND (TheLength > 0)
   AND (StartPos >= 0)
   AND (StartPos < TheLength) THEN
   BEGIN
      T := @TheStr[StartPos];
      S := Strings.StrPos(T,SubStr);
      IF S <> NIL THEN
         NextPos := StartPos+(S-T);
   END;
   FindNext := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindNextCh(Ch : CHAR;
                                  StartPos : WORD;
                                  VAR NextPos : WORD) : BOOLEAN;
VAR
   S,T : PChar;
BEGIN
   S := NIL;
   IF (TheLength > 0)
   AND (StartPos >= 0)
   AND (StartPos < TheLength) THEN
   BEGIN
      T := @TheStr[StartPos];
      S := Strings.StrScan(T,Ch);
      IF S <> NIL THEN
         NextPos := StartPos+(S-T);
   END;
   FindNextCh := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindPrev(SubStr      : PChar;
                                StartPos    : WORD;
                                VAR PrevPos : WORD) : BOOLEAN;
VAR
   S,T : PChar;
BEGIN
   S := NIL;
   IF (SafeStrLen(SubStr) > 0)
   AND (TheLength > 0)
   AND (StartPos >= 0)
   AND (StartPos < TheLength) THEN
   BEGIN
      PrevPos := StartPos;
      WHILE (S=NIL) AND (PrevPos >= 0) DO
      BEGIN
         T := @TheStr[PrevPos];
         S := Strings.StrPos(T,SubStr);
         IF S = NIL THEN
            DEC(PrevPos);
      END;
   END;
   FindPrev := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FindPrevCh(Ch          : CHAR;
                                  StartPos    : WORD;
                                  VAR PrevPos : WORD) : BOOLEAN;
VAR
   S,T : PChar;
BEGIN
   S := NIL;
   IF (TheLength > 0)
   AND (StartPos >= 0)
   AND (StartPos < TheLength) THEN
   BEGIN
      PrevPos := StartPos;
      WHILE (S=NIL) AND (PrevPos >= 0) DO
      BEGIN
         T := @TheStr[PrevPos];
         S := Strings.StrScan(T,Ch);
         IF S = NIL THEN
            DEC(PrevPos);
      END;
   END;
   FindPrevCh := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.HasCh(Ch : CHAR; VAR Pos : WORD) : BOOLEAN;
VAR
   P : PChar;
BEGIN
   IF TheLength = 0 THEN
      HasCh := FALSE
   ELSE
   BEGIN
      P := Strings.StrScan(TheStr,Ch);
      IF P <> NIL THEN
         Pos := P - TheStr;
      HasCh := (P <> NIL);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.IsChar(W : WORD; Ch : CHAR) : BOOLEAN;
BEGIN
   IF w >= TheLength THEN
      IsChar := FALSE
   ELSE
      IsChar := (TheStr[w] = Ch);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.GetChar(w : WORD) : CHAR;
BEGIN
   IF w >= TheLength THEN
      GetChar := #0
   ELSE
      GetChar := TheStr[w]
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.SetChar(W : WORD; Ch : CHAR);
BEGIN
   IF w < TheLength THEN
      TheStr[w] := Ch;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FirstCharIs(Ch : CHAR) : BOOLEAN;
BEGIN
   IF TheLength = 0 THEN
      FirstCharIs := FALSE
   ELSE
      FirstCharIs := (TheStr[0] = Ch);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FirstNonSpaceChar(VAR Ch : CHAR) : WORD;
VAR
   E : WORD;
BEGIN
   IF TheLength = 0 THEN
      FirstNonSpaceChar := WordResError
   ELSE
   BEGIN
      E := 0;
      WHILE TheStr[E] = ' ' DO
         INC(E);
      IF TheStr[E] = #0 THEN
         FirstNonSpaceChar := WordResError
      ELSE
      BEGIN
         Ch := TheStr[E];
         FirstNonSpaceChar := E;
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.LastNonSpaceChar(VAR Ch : CHAR) : WORD;
VAR
   E : WORD;
BEGIN
   IF TheLength = 0 THEN
      LastNonSpaceChar := WordResError
   ELSE
   BEGIN
      E := TheLength-1;
      WHILE (E > 0) AND (TheStr[E] = ' ') DO
         DEC(E);
      IF TheStr[E] = ' ' THEN
         LastNonSpaceChar := WordResError
      ELSE
      BEGIN
         Ch := TheStr[E];
         LastNonSpaceChar := E;
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.LastCharIs(Ch : CHAR) : BOOLEAN;
BEGIN
   IF TheLength = 0 THEN
      LastCharIs := FALSE
   ELSE
      LastCharIs := (TheStr[TheLength-1] = Ch);
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.RemoveLastChar;
BEGIN
   IF TheLength > 0 THEN
   BEGIN
      TheStr[TheLength-1] := #0;
      DEC(TheLength);
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceAll(OldStr,NewStr : PChar) : PChar;
VAR
   L,W,N,S : WORD;
BEGIN
   L := SafeStrLen(OldStr);
   IF L > 0 THEN
   BEGIN
      IF FindFirst(OldStr,w) THEN
      BEGIN
         Delete(w,L);
         N := SafeStrLen(NewStr);
         IF N > 0 THEN
            Insert(NewStr,w);
         s := w+N;
         WHILE FindNext(OldStr,S,w) DO
         BEGIN
            Delete(W,L);
            IF N > 0 THEN
               Insert(NewStr,W);
            s := W+N;
         END
      END
   END;
   ReplaceAll := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceFirst(OldStr,NewStr : PChar) : PChar;
VAR
   L,W : WORD;
BEGIN
   L := SafeStrLen(OldStr);
   IF L > 0 THEN
   BEGIN
      IF FindFirst(OldStr,w) THEN
      BEGIN
         Delete(w,L);
         Insert(NewStr,w);
      END
   END;
   ReplaceFirst := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceLast(OldStr,NewStr : PChar) : PChar;
VAR
   L,W : WORD;
BEGIN
   L := SafeStrLen(OldStr);
   IF L > 0 THEN
   BEGIN
      IF FindLast(OldStr,w) THEN
      BEGIN
         Delete(w,L);
         Insert(NewStr,w);
      END
   END;
   ReplaceLast := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceChAll(OldCh,NewCh : CHAR) : PChar;
VAR
   W,S : WORD;
BEGIN
   IF FindFirstCh(OldCh,w) THEN
   BEGIN
      TheStr[w] := NewCh;
      S := W+1;
      WHILE FindNextCh(OldCh,S,W) DO
      BEGIN
         TheStr[w] := NewCh;
         S := W+1;
      END
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceChFirst(OldCh,NewCh : CHAR) : PChar;
VAR
   W : WORD;
BEGIN
   IF FindFirstCh(OldCh,w) THEN
      TheStr[w] := NewCh;
   ReplaceChFirst := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ReplaceChLast(OldCh,NewCh : CHAR) : PChar;
VAR
   W : WORD;
BEGIN
   IF FindLastCh(OldCh,w) THEN
      TheStr[w] := NewCh;
   ReplaceChLast := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.SubStrCount(SubStr : PChar) : WORD;
VAR
   L,Ct,P,N : WORD;
BEGIN
   Ct := 0;
   L := SafeStrLen(SubStr);
   IF (L > 0) AND FindFirst(SubStr,P) THEN
   BEGIN
      INC(P,L);
      INC(Ct);
      WHILE FindNext(SubStr,P,N) DO
      BEGIN
         INC(Ct);
         P := N+L;
      END
   END;
   SubStrCount := Ct;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.FirstParseDelim(Source       : PChar;
                                       DelimCh      : CHAR;
                                       VAR DelimPos : WORD) : BOOLEAN;
VAR
   S : PChar;
   L : WORD;
BEGIN
   S := NIL;
   DelimPos := 0;
   L := SafeStrLen(Source);
   { null or nil source string? }
   IF L = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      S := Strings.StrScan(Source,DelimCh);
      IF S = NIL THEN
      BEGIN
         { no delimiter - but the string contains one item
           so return TRUE and copy that item into the
           object data string }
         Assign(Source);
         DelimPos := L;
         S := Source;
      END
      ELSE
      BEGIN
         DelimPos := S-Source;
         StrLCopy(Source,DelimPos);
      END
   END;
   FirstParseDelim := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.NextParseDelim(Source           : PChar;
                                      DelimCh          : CHAR;
                                      StartPos         : WORD;
                                      VAR NextDelimPos : WORD) : BOOLEAN;
VAR
   S,T : PChar;
   L : WORD;
BEGIN
   S := NIL;
   L := SafeStrLen(Source);
   { nothing left to process? }
   IF (L = 0) OR (StartPos <= 0) OR (StartPos >= L) THEN
      DisposeStr
   ELSE
   BEGIN
      T := @Source[StartPos];
      S := Strings.StrScan(T,DelimCh);
      { delimiter not found? }
      IF S = NIL THEN
      BEGIN
         { must be the last item, so copy into object }
         Assign(T);
         NextDelimPos := L;
         { return FALSE so as not to continue LOOP }
      END
      ELSE
      BEGIN
         StrLCopy(T,S-T);
         NextDelimPos := StartPos+(S-T);
      END
   END;
   NextParseDelim := (S <> NIL);
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ParseDelimCount(DelimCh : CHAR) : WORD;
BEGIN
   { add 1 as the text after last ',' counts as one delimited entry }
   ParseDelimCount := ChCount(DelimCh) + 1;
END;

{ ---------------------------------------------------------------------- }

{ Description : Parses the object's own data string 'TheStr' into
                component data items and stores each item as
                a 'PStringObject' object on the list . }

FUNCTION TStringObject.ParseDelimToList(DelimCh   : CHAR;
                                        Special   : INTEGER;
                                        VAR AList : PCollection) : WORD;
VAR
   AStrObj : PStringObject;
   DPos,NPos : WORD;
   F : BOOLEAN;
BEGIN
   IF AList = NIL THEN
      AList := NEW(PCollection,Init(10,10));
   AStrObj := NEW(PStringObject,InitDefault);
   F := AStrObj^.FirstParseDelim(TheStr,DelimCh,DPos);
   CASE Special OF
      delim_ExcDIC :
         AStrObj^.RemoveDIC;
      delim_ExcSIC :
         AStrObj^.RemoveSIC
   END;
   AList^.Insert(AStrObj);
   IF F THEN
   BEGIN
      INC(DPos);
      REPEAT
         AStrObj := NEW(PStringObject,InitDefault);
         F := AStrObj^.NextParseDelim(TheStr,DelimCh,DPos,NPos);
         AList^.Insert(AStrObj);
         CASE Special OF
            delim_ExcDIC :
               AStrObj^.RemoveDIC;
            delim_ExcSIC :
               AStrObj^.RemoveSIC
         END;
         DPos := NPos+1;
      UNTIL (NOT F);
   END;
   ParseDelimToList := AList^.Count;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ParsePosToList(VAR PosArray; PosCt : WORD; VAR AList : PCollection) : WORD;
CONST
   FirstChar = 1;
   CharCount = 2;
TYPE
   P2DimArray = ^T2DimArray;
   T2DimArray = ARRAY[1..1000,1..2] OF WORD;
VAR
   IArray : P2DimArray;
   E : INTEGER;
   AStrObj : PStringObject;
BEGIN
   IF PosCt > 0 THEN
   BEGIN
      IF AList = NIL THEN
         AList := NEW(PCollection,Init(20,20));
      IArray := @PosArray;
      FOR E := 1 TO PosCt DO
      BEGIN
         AStrObj := NEW(PStringObject,InitDefault);
         AStrObj^.AssignMid(GetPChar,IArray^[E,FirstChar],IArray^[E,CharCount]);
         AList^.Insert(AStrObj);
      END
   END;
   ParsePosToList := PosCt;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.PadCentre(NewLen : WORD; Ch : CHAR) : PChar;
VAR
   AStr : PChar;
   D,E : WORD;
BEGIN
   IF NewLen > TheLength THEN
   BEGIN
      D := (NewLen-TheLength) DIV 2;
      GetMem(AStr,D+1);
      FOR E := 0 TO D DO
         AStr[E] := Ch;
      AStr[D+1] := #0;
      Prepend(AStr);
      Append(AStr);
      FreeMem(AStr,D+1);
   END;
   PadCentre := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.PadEnd(NewLen : WORD; Ch : CHAR) : PChar;
VAR
   AStr : PChar;
   D,E : WORD;
BEGIN
   IF NewLen > TheLength THEN
   BEGIN
      D := NewLen-TheLength;
      GetMem(AStr,D+1);
      FOR E := 0 TO D DO
         AStr[E] := Ch;
      AStr[D+1] := #0;
      Append(AStr);
      FreeMem(AStr,D+1);
   END;
   PadEnd := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.PadFront(NewLen : WORD; Ch : CHAR) : PChar;
VAR
   AStr : PChar;
   D,E : WORD;
BEGIN
   IF NewLen > TheLength THEN
   BEGIN
      D := NewLen-TheLength;
      GetMem(AStr,D+1);
      FOR E := 0 TO D DO
         AStr[E] := Ch;
      AStr[D+1] := #0;
      Prepend(AStr);
      FreeMem(AStr,D+1);
   END;
   PadFront := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.Trim : PChar;
BEGIN
   TrimEnd;
   TrimFront;
   Trim := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.TrimEnd : PChar;
VAR
   E : WORD;
   ECh : CHAR;
BEGIN
   E := LastNonSpaceChar(ECh);
   IF (E <> WordResError) AND (E < (TheLength-1)) THEN
   BEGIN
      TheStr[E+1] := #0;
      TheLength := SafeStrLen(TheStr);
   END;
   TrimEnd := TheStr;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.TrimFront : PChar;
VAR
   E,S : WORD;
   ECh : CHAR;
BEGIN
   E := FirstNonSpaceChar(ECh);
   IF (E <> WordResError) AND (E > 0) THEN
   BEGIN
      S := E;
      WHILE TheStr[S] <> #0 DO
      BEGIN
         TheStr[S-E] := TheStr[S];
         INC(S);
      END;
      TheStr[S] := #0;
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.TrimZero : PChar;
VAR
   E : WORD;
BEGIN
   IF TheLength > 0 THEN
   BEGIN
      E := TheLength -1;
      WHILE (E > 0) AND (TheStr[E] = '0') DO
         DEC(E);
      IF E < (TheLength - 1) THEN
      BEGIN
         TheStr[E+1] := #0;
         TheLength := SafeStrLen(TheStr);
      END
   END;
   TrimZero := TheStr;
END;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

FUNCTION TStringObject.AssignString(S : STRING) : PChar;
VAR
   L : WORD;
   AStr : PChar;
BEGIN
   L := System.LENGTH(S);
   IF L = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      GetMem(AStr,L+1);
      Strings.StrPCopy(AStr,S);
      Assign(AStr);
      FreeMem(AStr,L+1);
   END;
   AssignString := TheStr;
END;

{ ---------------------------------------------------------------------- }

{ Description : this function returns TRUE if the string can accomadate
                a StrCat action of 'NewStr' into the string object's
                buffer. If the buffer is not large enough the function
                returns FALSE, with the 'Extra' variable to set to the
                amount of ADDITIONAL bytes required to accomadate the
                action. }

FUNCTION TStringObject.CanCat(NewStr : PChar; VAR Extra : WORD) : BOOLEAN;
VAR
   w : WORD;
BEGIN
   w := TheLength+SafeStrLen(NewStr);
   IF TheMaxSize >= w THEN
      Extra := 0
   ELSE
      Extra := ChkSizeInc(w - TheMaxSize);
   CanCat := (Extra <= 0);
END;

{ ---------------------------------------------------------------------- }

{ Description : this function returns TRUE if the string can accomadate
                a StrCopy action of 'NewStr' into the string object's
                buffer. If the buffer is not large enough the function
                returns FALSE, with the 'Extra' variable to set to the
                number of ADDITIONAL bytes required to accomadate the
                action. }

FUNCTION TStringObject.CanCopy(NewStr : PChar; VAR Extra : WORD) : BOOLEAN;
VAR
   w : WORD;
BEGIN
   w := SafeStrLen(NewStr);
   IF TheMaxSize >= w THEN
      Extra := 0
   ELSE
      Extra := ChkSizeInc(w-TheMaxSize);
   CanCopy := ((TheMaxSize > 0) AND (Extra <= 0));
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ChkSizeInc(E : WORD) : WORD;
BEGIN
   IF TheSizeInc <= E THEN
      ChkSizeInc := E
   ELSE
   BEGIN
      IF LONGINT(TheMaxSize+TheSizeInc+10) < $FFFF THEN
         ChkSizeInc := TheSizeInc
      ELSE
         ChkSizeInc := $FFFF - TheMaxSize - 10;
   END
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.CompConv(I : INTEGER) : INTEGER;
BEGIN
   IF I < 0 THEN
      CompConv := Compare_LT
   ELSE
   BEGIN
      IF I > 0 THEN
         CompConv := Compare_GT
      ELSE
         CompConv := Compare_EQ;
   END
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.DisposeStr;
BEGIN
   IF TheMaxSize > TheLength THEN
      FreeMem(TheStr,TheMaxSize+1)
   ELSE
      StrDispose(TheStr);
   TheStr := NIL;
   TheLength := 0;
   TheMaxSize := 0;
END;

{ ---------------------------------------------------------------------- }

FUNCTION TStringObject.ExpandBy(ExtraLen : WORD) : BOOLEAN;
VAR
   S : PChar;
BEGIN
   ExpandBy := FALSE;
   IF LONGINT(TheMaxSize+ExtraLen+1) > $FFFF THEN
      EXIT;
   GetMem(S,TheMaxSize+ExtraLen+1);
   IF TheStr <> NIL THEN
   BEGIN
      Strings.StrCopy(S,TheStr);
      FreeMem(TheStr,TheMaxSize+1);
   END;
   INC(TheMaxSize,ExtraLen);
   TheStr := S;
   ExpandBy := (TheStr <> NIL);
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.InitDataMembers;
BEGIN
   TheStr     := NIL;
   TheMaxSize := 0;
   TheLength  := 0;
   TheSizeInc := 0;
END;

{ ---------------------------------------------------------------------- }

PROCEDURE TStringObject.SetBufferLen(NewLen : WORD);
VAR
   NewStr : PChar;
BEGIN
   IF NewLen = 0 THEN
      DisposeStr
   ELSE
   BEGIN
      IF NewLen <> TheMaxSize THEN
      BEGIN
         GetMem(NewStr,NewLen+1);
         IF TheStr <> NIL THEN
         BEGIN
            Strings.StrLCopy(NewStr,TheStr,NewLen);
            IF TheMaxSize > TheLength THEN
               FreeMem(TheStr,TheMaxSize)
            ELSE
               Strings.StrDispose(TheStr);
            TheLength := SafeStrLen(NewStr);
         END;
         TheStr := NewStr;
         TheMaxSize := NewLen;
      END
   END
END;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

PROCEDURE FindSystemNumberChars;

   { +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ }

   PROCEDURE GetCharValue(VAR TheChar : CHAR;
                          DefChar     : CHAR;
                          KeyWord     : PChar);
   VAR
      AStr : ARRAY[0..20] OF CHAR;
   BEGIN
      AStr[0] := DefChar;
      AStr[1] := #0;
      GetProfileString('Intl',KeyWord,AStr,AStr,SIZEOF(AStr));
      IF SafeStrLen(AStr) = 0 THEN
         TheChar := DefChar
      ELSE
         TheChar := AStr[0];
   END;

   { +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ }

   PROCEDURE GetNumberValue(VAR ANumber: INTEGER;
                            DefNumber : INTEGER;
                            KeyWord : PChar);
   BEGIN
      ANumber := GetProfileInt('Intl',Keyword,DefNumber);
   END;

   { +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ }

BEGIN
   { find thousands character }
   GetCharValue(sThousand,',','sThousand');
   { find decimal character }
   GetCharValue(sDecimal,'.','sDecimal');
   { find decimal character }
   GetCharValue(sCurrency,'','sCurrency');
   { normal number of decimals }
   GetNumberValue(iCurrDigits,2,'iCurrDigits');
END;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

PROCEDURE InitUnit;
BEGIN
   {$IFDEF UseStreams}
   RegisterType(RStringObject);
   {$ENDIF}
   FindSystemNumberChars;
END;

{ ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }


BEGIN
   InitUnit;
END.